/******************************************************************************
*******************************************************************************
  ECE 270 Fall 2016

  Kernel HAL.h
	
  Christopher Theriault
  Homework #:5

  Copyright DigiPen (USA) Corporation
        All Rights Reserved
	
*******************************************************************************
******************************************************************************/

/******************************************************************************
*******************************************************************************
  References:

  [1] RM0090 Reference manual 
  STM32F405/415, STM32F407/417, STM32F427/437 and STM32F429/439 
    advanced ARM®-based 32-bit MCUs

  [2] UM1670 User manual
  Discovery kit with STM32F429ZI MCU

  [3] STM32F427xx/STM32F429xx Datasheet
  ARM Cortex-M4 32b MCU+FPU, 225DMIPS, up to 2MB Flash/256+4KB RAM, USB
    OTG HS/FS, Ethernet, 17 TIMs, 3 ADCs, 20 comm. interfaces, camera & LCD-TFT

  [4] PM0214 Programming manual
  STM32F3, STM32F4 and STM32L4 Series
    Cortex®-M4 programming manual

*******************************************************************************
******************************************************************************/

/*
        This code provides the Hardware Abstraction Layer (HAL) for the
    kernel.  This HAL only supports the STM32F429ZI microcontroller. 
*/

#ifndef		__KERNEL_HAL__H__
#define		__KERNEL_HAL__H__

/******************************************************************************
*******************************************************************************
    Includes
*******************************************************************************
******************************************************************************/
#include        "Kernel.h"

/******************************************************************************
*******************************************************************************
    Public Prototypes
*******************************************************************************
******************************************************************************/

/******************************************************************************
*******************************************************************************
    Declarations & Types
*******************************************************************************
******************************************************************************/
//  
//  [4] p.58, core_cmFunc.h in (Proj. Dir.)/CMSIS_4/CMSIS/Include
#define   OS_CRITICAL_BEGIN   asm("cpsid     i")
#define   OS_CRITICAL_END     asm("cpsie     i")
// The RED LED on the Discovery Board
#define   RED                 (0x01<<0x0E)
// The GREED LED on the Discovery Board
#define   GREEN               (0x01<<0x0D)    
       
#define   STATE_OFFSET        0x04

/******************************************************************************
    OS_SetLEDs
		
      Parameter is the bitwise OR of all the colors that should be set
    to ON.  This does not turn any LEDs OFF (requires OS_ClearLEDs).
******************************************************************************/
unsigned int
OS_SetLEDs (unsigned int);

/******************************************************************************
    OS_ClearLEDs
		
      Parameter is the bitwise OR of all the colors that should be set
    to OFF.  This does not turn any LEDs ON (requires OS_SetLEDs).
******************************************************************************/
unsigned int
OS_ClearLEDs (unsigned int);

/******************************************************************************
    OS_GetButton
		
      Returns nonzero if the button is pushed, otherwise returns zero. 
******************************************************************************/
unsigned int
OS_GetButton (void);

/******************************************************************************
    OS_InitKernelHAL
		
      Prepares the system hardware for use.
******************************************************************************/
unsigned int
OS_InitKernelHAL (void);

#endif	//	__KERNEL_HAL__H__

// EOF    Kernel HAL.h
// Note: Some IDEs generate warnings if a file doesn't end in whitespace,
//  but Embedded Studio doesn't seem to be one of them.
