/******************************************************************************
*******************************************************************************
  ECE 270 Fall 2016

  Kernel HAL_asm.s
	
  Christopher Theriault
  Homework #:5

  Copyright DigiPen (USA) Corporation
        All Rights Reserved
	
*******************************************************************************
******************************************************************************/

/******************************************************************************
      Contains gcc assembly directives that are too lengthy to try and include 
    in an inline string via a `.c` file
******************************************************************************/

/******************************************************************************
*******************************************************************************
  References:

  [1] Real-Time Operating Systems for
  ARM Cortex-M Microcontrollers

*******************************************************************************
******************************************************************************/

#include        "Kernel HAL_asm.h"

  .syntax unified                   // Required to execute Thumb-2 code

  .global OS_Start                  // Required to expose function to .c files
  .extern OS_TaskRUNNING            // Defined in Kernel.c
  .extern OS_TaskNEW                // Defined in Kernel.c

  .section .text                    // Where to locate this code
  .align  0

    /**************************************************************************
        NOTE: The code in OS_Start is largely derived from the code found in
      the Valvano textbook, page 177
    **************************************************************************/
// [1] p.177
  .thumb_func
OS_Start:

    ldr     r0, =OS_TaskRUNNING     // Pointer to TCB to execute              
    ldr     r2, [r0]                // R2 = *(OS_TaskRUNNING)                 
    ldr     sp, [r2]                // SP = *(TCB.sp)   
    ldr     r3, =OS_RUNNING
    str     r3, [r2, #TCB_STATE_OFFSET]	// State is upgraded to RUNNING
    pop     {r4-r11}                // Restore registers R4-R11               
    pop     {r0-r3}                 // Restore registers R0-R3                
    pop     {r12}                   // Restore register R12                   
    pop     {lr}                    // Restore the LR, but this value is      
                                    //	bogus as new tasks have no return     
    pop     {lr}                    // Place the starting address of the      
                                    //	task into the LR        
    pop     {r1}                    // Discards STATUS register (PSR)
    cpsie   i                       // Enable interrupts          
                
    bx      lr                      // Return to task & restore R0-R3, etc.   

    /**************************************************************************
        NOTE: The portion of the code in the Timer 6 ISR that directly pertains
      to context switching is based on the code found in the Valvano textbook, 
      page 176
    **************************************************************************/
  .thumb_func
  .weak TIM6_DAC_IRQHandler
TIM6_DAC_IRQHandler:
    /**************************************************************************
        Disable global interrupts and push R4 - R11 on the stack 
        [1] p.176
    **************************************************************************/
    cpsid   i                     // Disable interrupts
    push    {r4-r11}              // Save registers R4-R11
    /**************************************************************************
        Implement the code that clears the Timer 6 interrupt flag
    **************************************************************************/
TIM6_ISR:
    ldr     r0, =TIMER6_SR
    ldr     r1, [r0]
    bfc     r1, #0x00, #0x01
    str     r1, [r0]
TIM6_LOOP:
    ldr     r1, [r0]
    and     r1, r1, #TIMER_IF
    cmp     r1, #0x00
    bne     TIM6_LOOP
    /**************************************************************************
        Invoke OSp_ScheduleTask, taking care that the link register preserves
      the fact that it is in ISR mode
    **************************************************************************/
    push    {lr}
    bl      OSp_ScheduleTask
    pop     {lr}
    /**************************************************************************
        Complete the context switch based on the code from p. 176 in the
      Valvano textbook, while adding code that correctly toggles the states
      of the tasks between READY and RUNNING
    **************************************************************************/
    ldr     r0, =OS_TaskRUNNING   // Pointer to current TCB
    ldr     r1, [r0]              // R1 = OS_TaskRUNNING
    ldr     r3, =OS_READY
    str     r3, [r1, #TCB_STATE_OFFSET]	// State is degraded to READY
    str     sp, [r1]              // OS_TaskRUNNING->sp = SP
    ldr     r1, =OS_TaskNEW       // Pointer to new TCB
    ldr     r1, [r1]              // R1 = OS_TaskNEW
    ldr     r3, =OS_RUNNING
    str     r3, [r1, #TCB_STATE_OFFSET]	// State is upgraded to RUNNING
    str     r1, [r0]              // OS_TaskRUNNING = OS_TaskNEW
    ldr     sp, [r1]              // SP = OS_TaskNEW->sp
    pop     {r4-r11}              // Restore registers R4-R11
    cpsie   i                     // Enable interrupts
    bx      lr

// EOF    Kernel HALs.s
// Note: Some IDEs generate warnings if a file doesn't end in whitespace,
//  but Embedded Studio doesn't seem to be one of them.



